/*
 *  =======                                                     =======
 *  =======      ECI Hi-Focus ADSL USB Modem Linux driver       =======
 *  =======                                                     =======
 *  =======  (c) 18/06/2001 --> 03/07/2001 by Sebastien HELLEU  =======
 *  =======                                                     =======
 *  =======            web: http://flashcode.free.fr            =======
 *  =======              e-mail: s_helleu@yahoo.fr              =======
 *  =======                                                     =======
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */

/* eciusb.c - ECI Hi-Focus ADSL USB Modem Linux driver */


#include <stdio.h>

#include "eciusb.h"
#include "pusb.h"


#define DEBUG 1


#define ECI_OUT(MSG)        printf("eciusb: " MSG )

#ifdef DEBUG
#define ECI_DBG_OUT(MSG)    printf("eciusb: [DEBUG] " MSG )
#else
#define ECI_DBG_OUT(MSG)
#endif


int             verbose;
int             numbus, numdev;
pusb_device_t   pusb_dev;


// usage: displays Copyright and command line parameters

void usage()
{
    printf("\nECI Hi-Focus ADSL USB Modem Linux driver  (c) 23/06/2001  by Sebastien Helleu\n");
    printf("Special thanks to: Benoit Papillault, Jean-Sebastien Valette\n\n");
    printf("Syntax:  eciusb [-v]\n\n");
    printf("  -v  verbose\n\n");
    printf("Report any bug to :  s_helleu@yahoo.fr\n");
    printf("http://flashcode.free.fr\n");
    exit(1);
}

// ecamine_args : examine command line options

void examine_args(int argc, char *argv[])
{
    int     i;

    verbose = 0;
    for (i = 1; i < argc; i++)
        if (strcmp(argv[i], "-v") == 0)
            verbose = 1;
        else
            usage();
}

/* urb.c is file make with "extr_urb" (URBs extracted from your log file) */

#include "urb.c"

void test_envoi_URBs()
{
    unsigned char   buf[16], nom_urb[100];
    int             i, j;

    for (i = 0; i < NB_URB_DATA; i++)
    {
        sprintf(nom_urb, "URB %d", urb[i].num_urb);
        if (verbose)
            printf("sending %s... ", nom_urb);
        memcpy(buf, urb[i].buffer, urb[i].buf_size);
        if (pusb_control_msg(pusb_dev, urb[i].result[0], urb[i].result[1],
            urb[i].result[2], urb[i].result[3], buf, urb[i].buf_size, 100) < 0)
        {
            if (verbose)
                printf("KO!\n");
            perror(nom_urb);
        }
        else
        {
            if (verbose)
                printf("OK\n");
        }
    }
}

int main(int argc, char *argv[])
{
    examine_args(argc, argv);

    if (verbose)
        ECI_OUT("Scanning USB...\n");

    if ( (pusb_dev = pusb_search_open(ECI_VENDOR, ECI_PRODUCT)) == NULL)
    {
        ECI_OUT("ECI Hi-Focus ADSL USB modem not found...\n");
        return -1;
    }
    ECI_OUT("ECI Hi-Focus ADSL USB modem found !\n");

    if (pusb_claim_interface(pusb_dev, 0) < 0)
    {
        ECI_OUT("ECI Hi-Focus ADSL USB modem in use...\n");
        return -1;
    }

    if (pusb_set_configuration(pusb_dev, 1) < 0)
    {
        perror("pusb_set_configuration 1");
        return -1;
    }

    if (pusb_set_interface(pusb_dev, 0, 2) < 0)
    {
        perror("pusb_set_interface");
        return -1;
    }

    ECI_DBG_OUT("sending some URBs...\n");
    test_envoi_URBs();
    ECI_DBG_OUT("URBs sent.\n");

    ECI_DBG_OUT("sleeping 10 sec...\n");
    sleep(10);
    ECI_DBG_OUT("exiting...\n");

    if (pusb_release_interface(pusb_dev, 0) < 0)
    {
        perror("pusb_release_interface");
        return -1;
    }

    ECI_OUT("shutting down modem...\n");
    pusb_close(pusb_dev);
    return 0;
}
