/*
 *  =======                                                     =======
 *  =======      ECI Hi-Focus ADSL USB Modem Linux driver       =======
 *  =======                                                     =======
 *  =======  (c) 18/06/2001 --> 03/07/2001 by Sebastien HELLEU  =======
 *  =======                                                     =======
 *  =======            web: http://flashcode.free.fr            =======
 *  =======              e-mail: s_helleu@yahoo.fr              =======
 *  =======                                                     =======
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */

/* extr_urb.c - extract URBs from ECI Hi-Focus Windows log */


#include <stdio.h>
#include <string.h>
#include <sys/time.h>

#define MAX_URB     2000


int             nb_urb;
int             nb_urb_ok;
int             last_ok;

int             max_urb_size;

/* for each URB : */
int             size_urb[MAX_URB];
int             address_urb[MAX_URB];
unsigned char   *data_urb[MAX_URB];
unsigned char   *res_urb[MAX_URB];


/* read_log: analyze log file, looking for URB requests... */

int read_log(unsigned char *log_name)
{
    FILE            *log;
    unsigned char   line[1000], string[100], byte;
    char            *pos;
    int             i, urb_type_ok, nb_warn, nb_errors;

    if ( (log = fopen(log_name, "rt")) == NULL)
    {
        printf("Unable to open file '%s'\n", log_name);
        return -1;
    }
    printf("Analyzing log file...\n");
    nb_urb = 0;
    nb_warn = 0;
    nb_errors = 0;
    max_urb_size = 0;
    urb_type_ok = 0;
    while (!feof(log))
    {
        fgets(line, 1000, log);
        if (strstr(line, ">>>>>>> URB") != NULL)
        {
            nb_urb++;
            if (nb_urb > MAX_URB)
            {
                fclose(log);
                printf("FATAL ERROR: too mush URB.\n");
                printf("Please modify source (increase MAX_URB) and then run this program again.\n");
                printf("EXITING NOW !!!\n");
                return -1;
            }
            fgets(line, 1000, log);
            fgets(line, 1000, log);
            if (strstr(line, "-- URB_FUNCTION_VENDOR_DEVICE:") == NULL)
            {
                printf("URB %d (g.down): warning, function 'Vendor device' not found ! -- skipping URB\n", nb_urb);
                urb_type_ok = 0;
                nb_warn++;
            }
            else
            {
                fgets(line, 1000, log);
                fgets(line, 1000, log);
                if ( (pos = strstr(line, "TransferBufferLength =")) == NULL)
                {
                    printf("URB %d (g.down): error, buffer length not found ! -- skipping URB\n", nb_urb);
                    urb_type_ok = 0;
                    nb_errors++;
                }
                else
                {
                    strncpy(string, pos+23, 8);
                    string[8] = '\0';
                    sscanf(string, "%08x", &size_urb[nb_urb-1]);
                    fgets(line, 1000, log);
                    fgets(line, 1000, log);
                    fgets(line, 1000, log);
                    fgets(line, 1000, log);
                    if ( (pos = strstr(line, "0000: ")) == NULL)
                    {
                        printf("URB %d (g.down): error, data not found ! -- skipping URB\n", nb_urb);
                        urb_type_ok = 0;
                        nb_errors++;
                    }
                    else
                    {
                        data_urb[nb_urb-1] = (unsigned char *)malloc(size_urb[nb_urb-1]);
                        pos += 6;
                        string[2] = '\0';
                        for (i = 0; i < size_urb[nb_urb-1]; i++)
                        {
                            strncpy(string, pos, 2);
                            sscanf(string, "%02x", &byte);
                            data_urb[nb_urb-1][i] = byte;
                            pos += 3;
                        }
                        fgets(line, 1000, log);
                        fgets(line, 1000, log);
                        fgets(line, 1000, log);
                        fgets(line, 1000, log);
                        if ( (pos = strstr(line, "Value                   =")) == NULL)
                        {
                            printf("URB %d (g.down): error, address not found ! -- skipping URB", nb_urb);
                            urb_type_ok = 0;
                            nb_errors++;
                        }
                        else
                        {
                            strncpy(string, pos+26, 4);
                            string[4] = '\0';
                            sscanf(string, "%04x", &address_urb[nb_urb-1]);
                            urb_type_ok = 1;
                        }
                    }
                }
            }
        }
        if ( (urb_type_ok) && (strstr(line, "<<<<<<< URB") != NULL) )
        {
            fgets(line, 1000, log);
            fgets(line, 1000, log);
            if (strstr(line, "-- URB_FUNCTION_CONTROL_TRANSFER") == NULL)
            {
                printf("URB %d (c.back): error, function 'Control transfer' not found ! -- skipping URB\n", nb_urb);
                nb_errors++;
            }
            else
            {
                fgets(line, 1000, log);
                fgets(line, 1000, log);
                fgets(line, 1000, log);
                fgets(line, 1000, log);
                fgets(line, 1000, log);
                fgets(line, 1000, log);
                fgets(line, 1000, log);
                if ( (pos = strstr(line, "SetupPacket          :")) == NULL)
                {
                    printf("URB %d (coing back): error, setup packet not found ! -- skipping URB\n", nb_urb);
                    nb_errors++;
                }
                else
                {
                    pos += 23;
                    res_urb[nb_urb-1] = (unsigned char *)malloc(4);
                    string[2] = '\0';
                    for (i = 0; i < 4; i++)
                    {
                        strncpy(string, pos, 2);
                        sscanf(string, "%02x", &byte);
                        res_urb[nb_urb-1][i] = byte;
                        pos += 3;
                    }
                    if (size_urb[nb_urb-1] > max_urb_size)
                        max_urb_size = size_urb[nb_urb-1];
                    last_ok = nb_urb-1;
                }
            }
        }
    }
    nb_urb_ok = nb_urb-(nb_warn+nb_errors);
    printf("---> %d URBs found   (%d warnings, %d errors, %d OK)\n",
        nb_urb, nb_warn, nb_errors, nb_urb_ok);
    fclose(log);
    return 0;
}

/* write_C_data: writes C data */

int write_C_data(unsigned char *C_name, unsigned char *IHX_name)
{
    FILE            *C_src, *IHX_out;
    unsigned char   string[4096];
    int             i, j;
    time_t          my_time;

    if ( (C_src = fopen(C_name, "wt")) == NULL)
    {
        printf("Unable to create file '%s'\n", C_name);
        return -1;
    }

    if ( (IHX_out = fopen(IHX_name, "wt")) == NULL)
    {
        printf("Unable to create file '%s'\n", IHX_name);
        return -1;
    }

    /* source information */
    sprintf(string, "/* %s: include me in ECI Hi-Focus ADSL USB modem Linux driver ! */\n\n", C_name);
    fputs(string, C_src);
    time(&my_time);
    strcpy(string, ctime(&my_time));
    strcpy(string, "/* Made by EXTR_URB on ");
    sprintf(&string[strlen(string)], ctime(&my_time));
    string[strlen(string)-1] = '\0';
    strcpy(&string[strlen(string)], "      */\n");
    fputs(string, C_src);
    fputs("/* (c) 2001 by FlashCode - http://flashcode.free.fr  */\n\n", C_src);

    /* constants */
    sprintf(string, "#define MAX_URB_SIZE    %d\n", max_urb_size);
    fputs(string, C_src);
    sprintf(string, "#define NB_URB_DATA     %d\n\n", nb_urb_ok);
    fputs(string, C_src);

    /* urb_t type definition */
    fputs("typedef struct\n", C_src);
    fputs("{\n", C_src);
    fputs("    int             num_urb;\n", C_src);
    fputs("    unsigned char   buf_size;\n", C_src);
    fputs("    unsigned char   buffer[MAX_URB_SIZE];\n", C_src);
    fputs("    unsigned char   result[4];\n", C_src);
    fputs("} urb_t;\n\n", C_src);

    /* data... */
    fputs("urb_t   urb[NB_URB_DATA] = {\n", C_src);
    for (i = 0; i < nb_urb; i++)
    {
        if (res_urb[i] != NULL)
        {
            sprintf(string, "  /* URB %3d */ { %3d, %2d, { ", i+1, i+1, size_urb[i]);
            for (j = 0; j < size_urb[i]; j++)
            {
                sprintf(&string[strlen(string)], "0x%02X", data_urb[i][j]);
                if (j < size_urb[i]-1)
                    sprintf(&string[strlen(string)], ", ");
            }
            strcpy(&string[strlen(string)], " },\n");
            fputs(string, C_src);
            strcpy(string, "                           { ");
            for (j = 0; j < 4; j++)
            {
                sprintf(&string[strlen(string)], "0x%02X", res_urb[i][j]);
                if (j < 3)
                    sprintf(&string[strlen(string)], ", ");
            }
            if (i == last_ok)
                strcpy(&string[strlen(string)], " } } };\n");
            else
                strcpy(&string[strlen(string)], " } },\n");
            fputs(string, C_src);
        }
    }

    /* Firmware... */
    for (i = 0; i < nb_urb; i++)
    {
        if (res_urb[i] != NULL)
        {
            /* hexadecimal */
            sprintf(string, "%04X: ", address_urb[i]);
            for (j = 0; j < size_urb[i]; j++)
            {
                sprintf(&string[strlen(string)], "%02X", data_urb[i][j]);
                if (j < size_urb[i]-1)
                    sprintf(&string[strlen(string)], " ");
            }
            /* hexadecimal Intel */
            sprintf(string, ":%02X%04X00", size_urb[i], address_urb[i]);
            for (j = 0; j < size_urb[i]; j++)
                sprintf(&string[strlen(string)], "%02X", data_urb[i][j]);
            sprintf(&string[strlen(string)], "00");

            /* ascii */
            /*sprintf(&string[strlen(string)], "  >>>> ");
            for (j = 0; j < size_urb[i]; j++)
            {
                if ( (data_urb[i][j] < 32) || (data_urb[i][j] > 127) )
                    strcpy(&string[strlen(string)], ".");
                else
                    sprintf(&string[strlen(string)], "%c", data_urb[i][j]);
            }*/

            strcpy(&string[strlen(string)], " \n");
            fputs(string, IHX_out);
        }
    }

    printf("Creating file '%s' : succesful !\n", C_name);
    printf("Creating file '%s' : succesful !\n", IHX_name);
    fclose(C_src);
    return 0;
}

int main(int argc, char *argv[])
{
    if (argc != 4)
    {
        printf("\nEXTRACT URB  (c) 23/06/2001  by Sebastien Helleu\n");
        printf("This program is part of ECI Hi-Focus ADSL USB modem Linux driver\n");
        printf("(c) 2001 - Sebastien Helleu, Jean-Sebastien Valette, Benoit Papillault\n\n");
        printf("Extracts URBs from ECI Hi-Focus Windows log.\n\n");
        printf("  Syntax:  %s <log_file> <out_file_C> <out_file_IHX>\n\n", argv[0]);
        printf("Reads 'log_file' and produce 'out_file.c' (data for your C source)\n");
        printf("and 'out_file.ihx' (Intel Hexadecimal format)\n\n");
        printf("Note: 'out_file.c'   contains URB requests.\n");
        printf("      'out_file.ihx' contains ECI Hi-Focus Firmware with addresses.\n");
        return -1;
    }
    if (read_log(argv[1]) < 0)
        return -1;
    return write_C_data(argv[2], argv[3]);
}
